@@ -100,6 +100,7 @@ void FileSystem::setFileReadOnly(const QString &filename, bool readonly)
     if (!readonly) {
         permissions |= defaultWritePermissions;
     }
+    // Warning: This function does not manipulate ACLs, which may limit its effectiveness.
     file.setPermissions(permissions);
 }
 
@@ -197,6 +198,7 @@ bool FileSystem::uncheckedRenameReplace(const QString &originFileName,
 
 #else //Q_OS_WIN
     // You can not overwrite a read-only file on windows.
+
     if (!QFileInfo(destinationFileName).isWritable()) {
         setFileReadOnly(destinationFileName, false);
     }
@@ -362,6 +364,7 @@ bool FileSystem::remove(const QString &fileName, QString *errorString)
 #ifdef Q_OS_WIN
     // You cannot delete a read-only file on windows, but we want to
     // allow that.
+
     if (!QFileInfo(fileName).isWritable()) {
         setFileReadOnly(fileName, false);
     }
@@ -371,6 +374,7 @@ bool FileSystem::remove(const QString &fileName, QString *errorString)
         if (errorString) {
             *errorString = f.errorString();
         }
+        qCWarning(lcFileSystem) << "Failed to delete:" << fileName << "Error:" << f.errorString();
         return false;
     }
     return true;
@@ -484,6 +488,7 @@ bool FileSystem::isFileLocked(const QString &fileName, LockMode mode)
     }
 #else
     Q_UNUSED(fileName);
+    Q_UNUSED(mode);
 #endif
     return false;
 }
