@@ -37,14 +37,37 @@
 #include <QNetworkProxy>
 
 namespace {
-const char versionC[] = "version";
-const int maxFoldersVersion = 1;
+/*
+ * [Accounts]
+ * 0\version=1
+ */
+auto versionC()
+{
+    return QStringLiteral("version");
+}
+
+/*
+ * Folders with a version > maxFoldersVersion will be removed
+ * After the user was prompted for consent.
+ */
+constexpr int maxFoldersVersion = 1;
 
 int numberOfSyncJournals(const QString &path)
 {
     return QDir(path).entryList({ QStringLiteral(".sync_*.db"), QStringLiteral("._sync_*.db") }, QDir::Hidden | QDir::Files).size();
 }
 
+QString makeLegacyDbName(const OCC::FolderDefinition &def, const OCC::AccountPtr &account)
+{
+    // ensure https://demo.owncloud.org/ matches https://demo.owncloud.org
+    // the empty path was the legacy formating before 2.9
+    auto legacyUrl = account->url();
+    if (legacyUrl.path() == QLatin1String("/")) {
+        legacyUrl.setPath(QString());
+    }
+    const QString key = QStringLiteral("%1@%2:%3").arg(account->credentials()->user(), legacyUrl.toString(), def.targetPath);
+    return OCC::SyncJournalDb::makeDbName(def.localPath, QString::fromUtf8(QCryptographicHash::hash(key.toUtf8(), QCryptographicHash::Md5).left(6).toHex()));
+}
 }
 
 namespace OCC {
@@ -286,7 +309,26 @@ void FolderMan::setupFoldersHelper(QSettings &settings, AccountStatePtr account,
         FolderDefinition folderDefinition;
         settings.beginGroup(folderAlias);
         if (FolderDefinition::load(settings, folderAlias, &folderDefinition)) {
-            auto defaultJournalPath = folderDefinition.defaultJournalPath(account->account());
+            const auto defaultJournalPath = [&account, folderDefinition] {
+                // if we would have booth the 2.9.0 file name and the lagacy file
+                // with the md5 infix we prefer the 2.9.0 version
+                const QDir info(folderDefinition.localPath);
+                const QString defaultPath = SyncJournalDb::makeDbName(folderDefinition.localPath);
+                if (info.exists(defaultPath)) {
+                    return defaultPath;
+                }
+                // 2.6
+                QString legacyPath = makeLegacyDbName(folderDefinition, account->account());
+                if (info.exists(legacyPath)) {
+                    return legacyPath;
+                }
+                // pre 2.6
+                legacyPath.replace(QLatin1String(".sync_"), QLatin1String("._sync_"));
+                if (info.exists(legacyPath)) {
+                    return legacyPath;
+                }
+                return defaultPath;
+            }();
 
             // Migration: Old settings don't have journalPath
             if (folderDefinition.journalPath.isEmpty()) {
@@ -303,7 +345,7 @@ void FolderMan::setupFoldersHelper(QSettings &settings, AccountStatePtr account,
                 folderDefinition.journalPath = defaultJournalPath;
             }
 
-            // Migration: If an old db is found, move it to the new name.
+            // Migration: If an old .csync_journal.db is found, move it to the new name.
             if (backwardsCompatible) {
                 SyncJournalDb::maybeMigrateDb(folderDefinition.localPath, folderDefinition.absoluteJournalPath());
             }
@@ -317,7 +359,7 @@ void FolderMan::setupFoldersHelper(QSettings &settings, AccountStatePtr account,
             Folder *f = addFolderInternal(std::move(folderDefinition), account.data(), std::move(vfs));
             if (f) {
                 // Migrate the old "usePlaceholders" setting to the root folder pin state
-                if (settings.value(QLatin1String(versionC), 1).toInt() == 1
+                if (settings.value(versionC(), 1).toInt() == 1
                     && settings.value(QLatin1String("usePlaceholders"), false).toBool()) {
                     qCInfo(lcFolderMan) << "Migrate: From usePlaceholders to PinState::OnlineOnly";
                     f->setRootPinState(PinState::OnlineOnly);
@@ -329,6 +371,9 @@ void FolderMan::setupFoldersHelper(QSettings &settings, AccountStatePtr account,
                 if (foldersWithPlaceholders)
                     f->setSaveInFoldersWithPlaceholders();
 
+                // save possible changes from the migration
+                f->saveToSettings();
+
                 scheduleFolder(f);
                 emit folderSyncStateChange(f);
             }
@@ -370,12 +415,12 @@ void FolderMan::backwardMigrationSettingsKeys(QStringList *deleteKeys, QStringLi
 
     auto processSubgroup = [&](const QString &name) {
         settings->beginGroup(name);
-        const int foldersVersion = settings->value(QLatin1String(versionC), 1).toInt();
+        const int foldersVersion = settings->value(versionC(), 1).toInt();
         if (foldersVersion <= maxFoldersVersion) {
             const auto &childGroups = settings->childGroups();
             for (const auto &folderAlias : childGroups) {
                 settings->beginGroup(folderAlias);
-                const int folderVersion = settings->value(QLatin1String(versionC), 1).toInt();
+                const int folderVersion = settings->value(versionC(), 1).toInt();
                 if (folderVersion > FolderDefinition::maxSettingsVersion()) {
                     ignoreKeys->append(settings->group());
                 }
@@ -484,6 +529,10 @@ Folder *FolderMan::setupFolderFromOldConfigFile(const QString &file, AccountStat
     // check the unescaped variant (for the case when the filename comes out
     // of the directory listing). If the file does not exist, escape the
     // file and try again.
+
+#ifdef Q_OS_WIN
+    Utility::NtfsPermissionLookupRAII ntfs_perm;
+#endif
     QFileInfo cfgFile(_folderConfigPath, file);
 
     if (!cfgFile.exists()) {
@@ -616,8 +665,7 @@ void FolderMan::slotSyncOnceFileUnlocks(const QString &path, FileSystem::LockMod
   */
 void FolderMan::scheduleFolder(Folder *f)
 {
-    if (!f) {
-        qCCritical(lcFolderMan) << "slotScheduleSync called with null folder";
+    if (!OC_ENSURE(f)) {
         return;
     }
     auto alias = f->alias();
@@ -839,7 +887,7 @@ void FolderMan::slotStartScheduledFolderSync()
         registerFolderWithSocketApi(folder);
 
         _currentSyncFolder = folder;
-        folder->startSync(QStringList());
+        folder->startSync();
     }
 }
 
@@ -1011,7 +1059,7 @@ Folder *FolderMan::addFolder(AccountState *accountState, const FolderDefinition
 {
     // Choose a db filename
     auto definition = folderDefinition;
-    definition.journalPath = definition.defaultJournalPath(accountState->account());
+    definition.journalPath = SyncJournalDb::makeDbName(folderDefinition.localPath);
 
     if (!ensureJournalGone(definition.absoluteJournalPath())) {
         return nullptr;
@@ -1134,8 +1182,7 @@ QStringList FolderMan::findFileInLocalFolders(const QString &relPath, const Acco
 
 void FolderMan::removeFolder(Folder *f)
 {
-    if (!f) {
-        qCCritical(lcFolderMan) << "Can not remove null folder";
+    if (!OC_ENSURE(f)) {
         return;
     }
 
@@ -1265,7 +1312,7 @@ void FolderMan::setDirtyNetworkLimits()
 {
     for (auto *f : qAsConst(_folderMap)) {
         // set only in busy folders. Otherwise they read the config anyway.
-        if (f && f->isBusy()) {
+        if (f && f->isSyncRunning()) {
             f->setDirtyNetworkLimits();
         }
     }
@@ -1493,9 +1540,9 @@ Result<void, QString> FolderMan::unsupportedConfiguration(const QString &path) c
     return {};
 }
 
-bool FolderMan::checkVfsAvailability(const QString &path) const
+bool FolderMan::checkVfsAvailability(const QString &path, Vfs::Mode mode) const
 {
-    return unsupportedConfiguration(path) && Vfs::checkAvailability(path);
+    return unsupportedConfiguration(path) && Vfs::checkAvailability(path, mode);
 }
 
 } // namespace OCC
