@@ -25,6 +25,8 @@
 #include "navigationpanehelper.h"
 #include "syncfileitem.h"
 
+class TestFolderMigration;
+
 namespace OCC {
 namespace TestUtils {
     // prototype for test friend
@@ -240,7 +242,7 @@ public:
     void setDirtyNetworkLimits();
 
     /** Whether or not vfs is supported in the location. */
-    bool checkVfsAvailability(const QString &path) const;
+    bool checkVfsAvailability(const QString &path, Vfs::Mode mode = bestAvailableVfsMode()) const;
 
     /** If the folder configuration is no longer supported this will return an error string */
     Result<void, QString> unsupportedConfiguration(const QString &path) const;
@@ -387,6 +389,7 @@ private:
     explicit FolderMan(QObject *parent = nullptr);
     friend class OCC::Application;
     friend OCC::FolderMan *OCC::TestUtils::folderMan();
+    friend class ::TestFolderMigration;
 };
 
 } // namespace OCC
