@@ -48,7 +48,7 @@ ProtocolWidget::ProtocolWidget(QWidget *parent)
 
     connect(_ui->_tableView, &QTreeWidget::customContextMenuRequested, this, &ProtocolWidget::slotItemContextMenu);
 
-    _model = new ProtocolItemModel(this);
+    _model = new ProtocolItemModel(2000, false, this);
     _sortModel = new QSortFilterProxyModel(this);
     _sortModel->setSourceModel(_model);
     _sortModel->setSortRole(Models::UnderlyingDataRole);
@@ -59,6 +59,7 @@ ProtocolWidget::ProtocolWidget(QWidget *parent)
     header->setSectionResizeMode(QHeaderView::Interactive);
     header->setExpandingColumn(static_cast<int>(ProtocolItemModel::ProtocolItemRole::File));
     header->setSortIndicator(static_cast<int>(ProtocolItemModel::ProtocolItemRole::Time), Qt::DescendingOrder);
+    header->hideSection(static_cast<int>(ProtocolItemModel::ProtocolItemRole::Status));
     header->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(header, &QHeaderView::customContextMenuRequested, header, [header, this] {
         showHeaderContextMenu(header, _sortModel);
@@ -123,9 +124,11 @@ void ProtocolWidget::showContextMenu(QWidget *parent, ProtocolItemModel *model,
                 case SyncFileItem::SoftError:
                     Q_FALLTHROUGH();
                 case SyncFileItem::BlacklistedError:
-                    menu->addAction(tr("Retry sync"), parent, [&data] {
-                        data.folder()->journalDb()->wipeErrorBlacklistEntry(data.path());
-                        FolderMan::instance()->scheduleFolderNext(data.folder());
+                    menu->addAction(tr("Retry sync"), parent, [data, folder = QPointer<Folder>(data.folder())] {
+                        if (folder) {
+                            folder->journalDb()->wipeErrorBlacklistEntry(data.path());
+                            FolderMan::instance()->scheduleFolderNext(folder);
+                        }
                     });
                 default:
                     break;
