@@ -59,6 +59,7 @@
 #include <QJsonDocument>
 #include <QJsonObject>
 #include <QWidget>
+#include <QBuffer>
 
 #include <QClipboard>
 
@@ -449,17 +450,30 @@ void SocketApi::slotUpdateFolderView(Folder *f)
 
     if (f) {
         // do only send UPDATE_VIEW for a couple of status
-        if (f->syncResult().status() == SyncResult::SyncPrepare
-            || f->syncResult().status() == SyncResult::Success
-            || f->syncResult().status() == SyncResult::Paused
-            || f->syncResult().status() == SyncResult::Problem
-            || f->syncResult().status() == SyncResult::Error
-            || f->syncResult().status() == SyncResult::SetupError) {
-            QString rootPath = removeTrailingSlash(f->path());
+        switch (f->syncResult().status()) {
+        case SyncResult::SyncPrepare:
+            Q_FALLTHROUGH();
+        case SyncResult::Success:
+            Q_FALLTHROUGH();
+        case SyncResult::Paused:
+            Q_FALLTHROUGH();
+        case SyncResult::Problem:
+            Q_FALLTHROUGH();
+        case SyncResult::Error:
+            Q_FALLTHROUGH();
+        case SyncResult::SetupError: {
+            const QString rootPath = removeTrailingSlash(f->path());
             broadcastStatusPushMessage(rootPath, f->syncEngine().syncFileStatusTracker().fileStatus(""));
 
-            broadcastMessage(buildMessage(QLatin1String("UPDATE_VIEW"), rootPath));
-        } else {
+            broadcastMessage(buildMessage(QStringLiteral("UPDATE_VIEW"), rootPath));
+        } break;
+        case OCC::SyncResult::Undefined:
+            Q_FALLTHROUGH();
+        case OCC::SyncResult::NotYetStarted:
+            Q_FALLTHROUGH();
+        case OCC::SyncResult::SyncRunning:
+            Q_FALLTHROUGH();
+        case OCC::SyncResult::SyncAbortRequested:
             qCDebug(lcSocketApi) << "Not sending UPDATE_VIEW for" << f->alias() << "because status() is" << f->syncResult().status();
         }
     }
@@ -536,18 +550,17 @@ void SocketApi::command_RETRIEVE_FILE_STATUS(const QString &argument, SocketList
     auto fileData = FileData::get(argument);
     if (!fileData.folder) {
         // this can happen in offline mode e.g.: nothing to worry about
-        statusString = QLatin1String("NOP");
+        statusString = SyncFileStatus(SyncFileStatus::StatusNone).toSocketAPIString();
     } else {
         // The user probably visited this directory in the file shell.
         // Let the listener know that it should now send status pushes for sibblings of this file.
         QString directory = fileData.localPath.left(fileData.localPath.lastIndexOf('/'));
         listener->registerMonitoredDirectory(qHash(directory));
 
-        SyncFileStatus fileStatus = fileData.syncFileStatus();
-        statusString = fileStatus.toSocketAPIString();
+        statusString = fileData.syncFileStatus().toSocketAPIString();
     }
 
-    const QString message = QLatin1String("STATUS:") % statusString % QLatin1Char(':') % QDir::toNativeSeparators(argument);
+    const QString message = QStringLiteral("STATUS:") % statusString % QLatin1Char(':') % QDir::toNativeSeparators(argument);
     listener->sendMessage(message);
 }
 
@@ -696,7 +709,6 @@ void SocketApi::fetchPrivateLinkUrlHelper(const QString &localFile, const std::f
     fetchPrivateLinkUrl(
         fileData.folder->accountState()->account(),
         fileData.serverRelativePath,
-        record.legacyDeriveNumericFileId(),
         this,
         targetFun);
 }
@@ -739,7 +751,7 @@ void SocketApi::command_MAKE_AVAILABLE_LOCALLY(const QString &filesArg, SocketLi
 
     for (const auto &file : files) {
         auto data = FileData::get(file);
-        if (!data.folder)
+        if (!data.folder || !data.folder->isReady())
             continue;
 
         // Update the pin state on all items
@@ -758,7 +770,7 @@ void SocketApi::command_MAKE_ONLINE_ONLY(const QString &filesArg, SocketListener
 
     for (const auto &file : files) {
         auto data = FileData::get(file);
-        if (!data.folder)
+        if (!data.folder || !data.folder->isReady())
             continue;
 
         // Update the pin state on all items
@@ -837,7 +849,8 @@ void SocketApi::command_V2_LIST_ACCOUNTS(const QSharedPointer<SocketApiJobV2> &j
     QJsonArray out;
     for (auto acc : AccountManager::instance()->accounts()) {
         out << QJsonObject({ { QStringLiteral("name"), acc->account()->displayName() },
-            { QStringLiteral("id"), acc->account()->id() } });
+            { QStringLiteral("id"), acc->account()->id() },
+            { QStringLiteral("uuid"), acc->account()->uuid().toString(QUuid::WithoutBraces) } });
     }
     job->success({ { QStringLiteral("accounts"), out } });
 }
@@ -848,6 +861,47 @@ void SocketApi::command_V2_UPLOAD_FILES_FROM(const QSharedPointer<SocketApiJobV2
     uploadJob->start();
 }
 
+void SocketApi::command_V2_GET_CLIENT_ICON(const QSharedPointer<SocketApiJobV2> &job) const
+{
+    OC_ASSERT(job);
+    const auto &arguments = job->arguments();
+
+    const auto size = arguments.value("size");
+    if (size.isUndefined()) {
+        qCWarning(lcSocketApi) << "Icon size not given in " << Q_FUNC_INFO;
+        job->failure(QStringLiteral("cannot get client icon"));
+        return;
+    }
+
+    const Theme *theme = Theme::instance();
+    OC_ASSERT(theme);
+    const QIcon appIcon = theme->applicationIcon();
+    qCDebug(lcSocketApi) << Q_FUNC_INFO << " got icon from theme: " << appIcon;
+
+    // convert to pixmap (might be smaller if size is not available)
+    const QPixmap pixmap = appIcon.pixmap(QSize(size.toInt(), size.toInt()));
+
+    // Convert pixmap to in-memory PNG
+    QByteArray png;
+    QBuffer pngBuffer(&png);
+    auto success = pngBuffer.open(QIODevice::WriteOnly);
+    if (!success) {
+        qCWarning(lcSocketApi) << "Error opening buffer for png in " << Q_FUNC_INFO;
+        job->failure(QStringLiteral("cannot get client icon"));
+        return;
+    }
+
+    success = pixmap.save(&pngBuffer, "PNG");
+    if (!success) {
+        qCWarning(lcSocketApi) << "Error saving client icon as png in " << Q_FUNC_INFO;
+        job->failure(QStringLiteral("cannot get client icon"));
+        return;
+    }
+
+    const QByteArray pngAsBase64 = pngBuffer.data().toBase64();
+    job->success({ { QStringLiteral("png"), QString::fromUtf8(pngAsBase64) } });
+}
+
 void SocketApi::emailPrivateLink(const QString &link)
 {
     Utility::openEmailComposer(
@@ -882,7 +936,7 @@ void SocketApi::sendSharingContextMenuOptions(const FileData &fileData, SocketLi
 {
     auto record = fileData.journalRecord();
     bool isOnTheServer = record.isValid();
-    auto flagString = isOnTheServer ? QLatin1String("::") : QLatin1String(":d:");
+    auto flagString = isOnTheServer ? QStringLiteral("::") : QStringLiteral(":d:");
 
     auto capabilities = fileData.folder->accountState()->account()->capabilities();
     auto theme = Theme::instance();
@@ -892,9 +946,9 @@ void SocketApi::sendSharingContextMenuOptions(const FileData &fileData, SocketLi
     // If sharing is globally disabled, do not show any sharing entries.
     // If there is no permission to share for this file, add a disabled entry saying so
     if (isOnTheServer && !record._remotePerm.isNull() && !record._remotePerm.hasPermission(RemotePermissions::CanReshare)) {
-        listener->sendMessage(QLatin1String("MENU_ITEM:DISABLED:d:") + (!record.isDirectory() ? tr("Resharing this file is not allowed") : tr("Resharing this folder is not allowed")));
+        listener->sendMessage(QStringLiteral("MENU_ITEM:DISABLED:d:") + (!record.isDirectory() ? tr("Resharing this file is not allowed") : tr("Resharing this folder is not allowed")));
     } else {
-        listener->sendMessage(QLatin1String("MENU_ITEM:SHARE") + flagString + tr("Share..."));
+        listener->sendMessage(QStringLiteral("MENU_ITEM:SHARE") + flagString + tr("Share..."));
 
         // Do we have public links?
         bool publicLinksEnabled = theme->linkSharing() && capabilities.sharePublicLink();
@@ -904,13 +958,13 @@ void SocketApi::sendSharingContextMenuOptions(const FileData &fileData, SocketLi
             && !capabilities.sharePublicLinkEnforcePasswordForReadOnly();
 
         if (canCreateDefaultPublicLink) {
-            listener->sendMessage(QLatin1String("MENU_ITEM:COPY_PUBLIC_LINK") + flagString + tr("Copy public link to clipboard"));
+            listener->sendMessage(QStringLiteral("MENU_ITEM:COPY_PUBLIC_LINK") + flagString + tr("Create and copy public link to clipboard"));
         } else if (publicLinksEnabled) {
-            listener->sendMessage(QLatin1String("MENU_ITEM:MANAGE_PUBLIC_LINKS") + flagString + tr("Copy public link to clipboard"));
+            listener->sendMessage(QStringLiteral("MENU_ITEM:MANAGE_PUBLIC_LINKS") + flagString + tr("Copy public link to clipboard"));
         }
     }
 
-    listener->sendMessage(QLatin1String("MENU_ITEM:COPY_PRIVATE_LINK") + flagString + tr("Copy private link to clipboard"));
+    listener->sendMessage(QStringLiteral("MENU_ITEM:COPY_PRIVATE_LINK") + flagString + tr("Copy private link to clipboard"));
 
     // Disabled: only providing email option for private links would look odd,
     // and the copy option is more general.
@@ -930,21 +984,18 @@ SocketApi::FileData SocketApi::FileData::get(const QString &localFile)
         return data;
 
     data.serverRelativePath = QDir(data.folder->remotePath()).filePath(data.folderRelativePath);
-    QString virtualFileExt = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
-    if (data.serverRelativePath.endsWith(virtualFileExt)) {
-        data.serverRelativePath.chop(virtualFileExt.size());
+    if (data.folder->isReady()) {
+        data.serverRelativePath = data.folder->vfs().underlyingFileName(data.serverRelativePath);
     }
     return data;
 }
 
 QString SocketApi::FileData::folderRelativePathNoVfsSuffix() const
 {
-    auto result = folderRelativePath;
-    QString virtualFileExt = QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
-    if (result.endsWith(virtualFileExt)) {
-        result.chop(virtualFileExt.size());
+    if (folder->isReady()) {
+        return folder->vfs().underlyingFileName(folderRelativePath);
     }
-    return result;
+    return folderRelativePath;
 }
 
 SyncFileStatus SocketApi::FileData::syncFileStatus() const
@@ -1056,6 +1107,7 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
 
     // File availability actions
     if (folder
+        && folder->isReady()
         && folder->virtualFilesEnabled()
         && folder->vfs().socketApiPinStateActionsShown()) {
         OC_ENFORCE(!files.isEmpty());
@@ -1074,6 +1126,9 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
             return VfsItemAvailability::Mixed;
         };
         for (const auto &file : files) {
+            if (!folder->isReady()) {
+                continue;
+            }
             auto fileData = FileData::get(file);
             auto availability = folder->vfs().availability(fileData.folderRelativePath);
             if (!availability) {
@@ -1091,13 +1146,14 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
 
         // TODO: Should be a submenu, should use icons
         auto makePinContextMenu = [&](bool makeAvailableLocally, bool freeSpace) {
-            listener->sendMessage(QLatin1String("MENU_ITEM:CURRENT_PIN:d:")
+            listener->sendMessage(QStringLiteral("MENU_SEPARATOR:d::"));
+            listener->sendMessage(QStringLiteral("MENU_ITEM:CURRENT_PIN:d:")
                 + Utility::vfsCurrentAvailabilityText(*combined));
-            listener->sendMessage(QLatin1String("MENU_ITEM:MAKE_AVAILABLE_LOCALLY:")
-                + (makeAvailableLocally ? QLatin1String(":") : QLatin1String("d:"))
+            listener->sendMessage(QStringLiteral("MENU_ITEM:MAKE_AVAILABLE_LOCALLY:")
+                + (makeAvailableLocally ? QStringLiteral(":") : QStringLiteral("d:"))
                 + Utility::vfsPinActionText());
-            listener->sendMessage(QLatin1String("MENU_ITEM:MAKE_ONLINE_ONLY:")
-                + (freeSpace ? QLatin1String(":") : QLatin1String("d:"))
+            listener->sendMessage(QStringLiteral("MENU_ITEM:MAKE_ONLINE_ONLY:")
+                + (freeSpace ? QStringLiteral(":") : QStringLiteral("d:"))
                 + Utility::vfsFreeSpaceActionText());
         };
 
@@ -1118,7 +1174,7 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
         }
     }
 
-    listener->sendMessage(QString("GET_MENU_ITEMS:END"));
+    listener->sendMessage(QStringLiteral("GET_MENU_ITEMS:END"));
 }
 
 #if GUI_TESTING
