@@ -12,45 +12,51 @@
  * for more details.
  */
 
-#include <QDesktopServices>
-#include <QNetworkReply>
-#include <QTimer>
-#include <QBuffer>
-#include "account.h"
 #include "creds/oauth.h"
-#include <QJsonObject>
-#include <QJsonDocument>
-#include "theme.h"
-#include "networkjobs.h"
+
+#include "account.h"
+#include "credentialmanager.h"
 #include "creds/httpcredentials.h"
+#include "networkjobs.h"
+#include "theme.h"
+
+#include <QApplication>
+#include <QBuffer>
+#include <QDesktopServices>
+#include <QJsonArray>
+#include <QJsonDocument>
+#include <QJsonObject>
+#include <QNetworkReply>
 #include <QRandomGenerator>
+#include <QScopeGuard>
+#include <QTimer>
 
-namespace OCC {
-
+using namespace OCC;
 Q_LOGGING_CATEGORY(lcOauth, "sync.credentials.oauth", QtInfoMsg)
 
-OAuth::OAuth(Account *account, QObject *parent)
-: QObject(parent)
-, _account(account)
+namespace {
+const QString clientSecretC()
 {
+    return QStringLiteral("http/clientSecret");
 }
 
-OAuth::~OAuth()
+QVariant getRequiredField(const QVariantMap &json, const QString &s, QString *error)
 {
+    const auto out = json.constFind(s);
+    if (out == json.constEnd()) {
+        error->append(QStringLiteral("\tError: Missing field %1\n").arg(s));
+        return {};
+    }
+    return *out;
 }
 
-static void httpReplyAndClose(QPointer<QTcpSocket> socket, const QByteArray &code, const QByteArray &html,
-                              const QByteArray &moreHeaders = {})
+void httpReplyAndClose(const QPointer<QTcpSocket> &socket, const QByteArray &code, const QByteArray &html,
+    const QByteArray &moreHeaders = {})
 {
     if (!socket)
         return; // socket can have been deleted if the browser was closed
-    const QByteArray msg = QByteArrayLiteral("HTTP/1.1 ") %
-            code %
-            QByteArrayLiteral("\r\nContent-Type: text/html; charset=utf-8\r\nConnection: close\r\nContent-Length: ") %
-            QByteArray::number(html.length()) %
-            (!moreHeaders.isEmpty() ? QByteArrayLiteral("\r\n") % moreHeaders : QByteArray()) %
-            QByteArrayLiteral("\r\n\r\n") %
-            html;
+    // clang format has issues with the next line...
+    const QByteArray msg = QByteArrayLiteral("HTTP/1.1 ") % code % QByteArrayLiteral("\r\nContent-Type: text/html; charset=utf-8\r\nConnection: close\r\nContent-Length: ") % QByteArray::number(html.length()) % (!moreHeaders.isEmpty() ? QByteArrayLiteral("\r\n") % moreHeaders : QByteArray()) % QByteArrayLiteral("\r\n\r\n") % html;
     qCDebug(lcOauth) << msg;
     socket->write(msg);
     socket->disconnectFromHost();
@@ -59,6 +65,113 @@ static void httpReplyAndClose(QPointer<QTcpSocket> socket, const QByteArray &cod
     socket->setParent(nullptr);
 }
 
+class RegisterClientJob : public QObject
+{
+    Q_OBJECT
+public:
+    RegisterClientJob(Account *accout, const QUrl &registrationEndpoint, QObject *parent)
+        : QObject(parent)
+        , _account(accout)
+        , _registrationEndpoint(registrationEndpoint)
+    {
+        connect(this, &RegisterClientJob::errorOccured, this, &RegisterClientJob::deleteLater);
+        connect(this, &RegisterClientJob::finished, this, &RegisterClientJob::deleteLater);
+    }
+
+    void start()
+    {
+        auto job = _account->credentialManager()->get(clientSecretC());
+        connect(job, &CredentialJob::finished, this, [this, job] {
+            qDebug() << job->errorString() << job->error();
+            if (job->data().isValid()) {
+                rgisterClientFinished(job->data().value<QVariantMap>());
+            } else {
+                qCCritical(lcOauth) << "Failed to read client id" << job->errorString();
+                rgisterClientOnline();
+            }
+        });
+    }
+
+Q_SIGNALS:
+    void finished(const QString &clientId, const QString &clientSecret);
+    void errorOccured(const QString &error);
+
+private:
+    void rgisterClientOnline()
+    {
+        auto job = new OCC::SimpleNetworkJob(_account->sharedFromThis(), this);
+        job->setAuthenticationJob(true);
+        connect(job, &OCC::SimpleNetworkJob::finishedSignal, this, [this, job] {
+            const auto data = job->reply()->readAll();
+            QJsonParseError error;
+            const auto json = QJsonDocument::fromJson(data, &error);
+            if (error.error == QJsonParseError::NoError) {
+                rgisterClientFinished(json.object().toVariantMap());
+            } else {
+                qCWarning(lcOauth) << "Failed to register the client" << error.errorString() << data;
+                Q_EMIT errorOccured(error.errorString());
+            }
+        });
+        const QJsonObject json({ { QStringLiteral("client_name"), QStringLiteral("%1 %2").arg(Theme::instance()->appNameGUI(), Theme::instance()->version()) },
+            { QStringLiteral("redirect_uris"), QJsonArray { QStringLiteral("http://127.0.0.1") } },
+            { QStringLiteral("application_type"), QStringLiteral("native") },
+            { QStringLiteral("token_endpoint_auth_method"), QStringLiteral("client_secret_basic") } });
+        job->prepareRequest("POST", _registrationEndpoint, QNetworkRequest(), json);
+        job->start();
+    }
+
+    void rgisterClientFinished(const QVariantMap &data)
+    {
+        {
+            QString error;
+            const auto expireDate = getRequiredField(data, QStringLiteral("client_secret_expires_at"), &error).value<qint64>();
+            if (!error.isEmpty()) {
+                Q_EMIT errorOccured(error);
+                return;
+            }
+            // 0 means it doesn't expire
+            if (expireDate) {
+                const auto qExpireDate = QDateTime::fromSecsSinceEpoch(expireDate);
+                qCInfo(lcOauth) << "Client id iessued at:" << QDateTime::fromSecsSinceEpoch(data[QStringLiteral("client_id_issued_at")].value<quint64>())
+                                << "expires at" << qExpireDate;
+                if (QDateTime::currentDateTimeUtc() > qExpireDate) {
+                    qCDebug(lcOauth) << "Client registration expired";
+                    rgisterClientOnline();
+                    return;
+                }
+            }
+        }
+        _account->credentialManager()->set(clientSecretC(), data);
+        QString error;
+        const auto client_id = getRequiredField(data, QStringLiteral("client_id"), &error).toString();
+        const auto client_secret = getRequiredField(data, QStringLiteral("client_secret"), &error).toString();
+        if (!error.isEmpty()) {
+            Q_EMIT errorOccured(error);
+            return;
+        }
+        Q_EMIT finished(client_id, client_secret);
+    }
+
+private:
+    Account *_account;
+    QUrl _registrationEndpoint;
+};
+
+}
+
+OAuth::OAuth(Account *account, QObject *parent)
+    : QObject(parent)
+    , _account(account)
+    , _clientId(Theme::instance()->oauthClientId())
+    , _clientSecret(Theme::instance()->oauthClientSecret())
+    , _redirectUrl(Theme::instance()->oauthLocalhost())
+{
+}
+
+OAuth::~OAuth()
+{
+}
+
 void OAuth::startAuthentication()
 {
     // Listen on the socket to get a port which will be used in the redirect_uri
@@ -71,8 +184,22 @@ void OAuth::startAuthentication()
     OC_ASSERT(_pkceCodeVerifier.size() == 128)
     _state = generateRandomString(8);
 
-    connect(this, &OAuth::fetchWellKnownFinished, this, [this]{
-        Q_EMIT authorisationLinkChanged(authorisationLink());
+    connect(this, &OAuth::fetchWellKnownFinished, this, [this] {
+        if (_registrationEndpoint.isValid()) {
+            auto job = new RegisterClientJob(_account, _registrationEndpoint, this);
+            connect(job, &RegisterClientJob::finished, this, [this](const QString &clientId, const QString &clientSecret) {
+                _clientId = clientId;
+                _clientSecret = clientSecret;
+                Q_EMIT authorisationLinkChanged(authorisationLink());
+            });
+            connect(job, &RegisterClientJob::errorOccured, this, [this](const QString &error) {
+                qCWarning(lcOauth) << "Failed to dynamically register the client, try the default client id" << error;
+                Q_EMIT authorisationLinkChanged(authorisationLink());
+            });
+            job->start();
+        } else {
+            Q_EMIT authorisationLinkChanged(authorisationLink());
+        }
     });
     fetchWellKnown();
 
@@ -97,31 +224,35 @@ void OAuth::startAuthentication()
                     httpReplyAndClose(socket, QByteArrayLiteral("400 Bad Request"), QByteArrayLiteral("<html><head><title>400 Bad Request</title></head><body><center><h1>400 Bad Request</h1></center></body></html>"));
                     return;
                 }
-              auto job = postTokenRequest({
+                auto job = postTokenRequest({
                     { QStringLiteral("grant_type"), QStringLiteral("authorization_code") },
                     { QStringLiteral("code"), args.queryItemValue(QStringLiteral("code")) },
-                    { QStringLiteral("redirect_uri"), QStringLiteral("http://localhost:%1").arg(_server.serverPort()) },
+                    { QStringLiteral("redirect_uri"), QStringLiteral("%1:%2").arg(_redirectUrl, QString::number(_server.serverPort())) },
                     { QStringLiteral("code_verifier"), QString::fromUtf8(_pkceCodeVerifier) },
-                    });
+                });
                 QObject::connect(job, &SimpleNetworkJob::finishedSignal, this, [this, socket](QNetworkReply *reply) {
+                    qScopeGuard([this] {
+                        // close the server once we are done here
+                        _server.close();
+                    });
                     const auto jsonData = reply->readAll();
                     QJsonParseError jsonParseError;
-                    const QJsonObject json = QJsonDocument::fromJson(jsonData, &jsonParseError).object();
+                    const auto data = QJsonDocument::fromJson(jsonData, &jsonParseError).object().toVariantMap();
                     QString fieldsError;
-                    const QString accessToken = getRequiredField(json, QStringLiteral("access_token"), &fieldsError).toString();
-                    const QString refreshToken = getRequiredField(json, QStringLiteral("refresh_token"), &fieldsError).toString();
-                    const QString tokenType = getRequiredField(json, QStringLiteral("token_type"), &fieldsError).toString().toLower();
-                    const QString user = json[QStringLiteral("user_id")].toString();
-                    const QUrl messageUrl = QUrl::fromEncoded(json[QStringLiteral("message_url")].toString().toUtf8());
+                    const QString accessToken = getRequiredField(data, QStringLiteral("access_token"), &fieldsError).toString();
+                    const QString refreshToken = getRequiredField(data, QStringLiteral("refresh_token"), &fieldsError).toString();
+                    const QString tokenType = getRequiredField(data, QStringLiteral("token_type"), &fieldsError).toString().toLower();
+                    const QString user = data[QStringLiteral("user_id")].toString();
+                    const QUrl messageUrl = QUrl::fromEncoded(data[QStringLiteral("message_url")].toByteArray());
 
                     if (reply->error() != QNetworkReply::NoError || jsonParseError.error != QJsonParseError::NoError
                         || !fieldsError.isEmpty()
                         || tokenType != QLatin1String("bearer")) {
                         // do we have error message suitable for users?
-                        QString errorReason = json[QStringLiteral("error_description")].toString();
+                        QString errorReason = data[QStringLiteral("error_description")].toString();
                         if (errorReason.isEmpty()) {
                             // fall back to technical error
-                            errorReason = json[QStringLiteral("error")].toString();
+                            errorReason = data[QStringLiteral("error")].toString();
                         }
                         if (!errorReason.isEmpty()) {
                             errorReason = tr("Error returned from the server: <em>%1</em>")
@@ -129,12 +260,6 @@ void OAuth::startAuthentication()
                         } else if (reply->error() != QNetworkReply::NoError) {
                             errorReason = tr("There was an error accessing the 'token' endpoint: <br><em>%1</em>")
                                               .arg(reply->errorString().toHtmlEscaped());
-                        } else if (jsonData.isEmpty()) {
-                            // Can happen if a funky load balancer strips away POST data, e.g. BigIP APM my.policy
-                            errorReason = tr("Empty JSON from OAuth2 redirect");
-                            // We explicitly have this as error case since the json qcWarning output below is misleading,
-                            // it will show a fake json will null values that actually never was received like this as
-                            // soon as you access json["whatever"] the debug output json will claim to have "whatever":null
                         } else if (jsonParseError.error != QJsonParseError::NoError) {
                             errorReason = tr("Could not parse the JSON returned from the server: <br><em>%1</em>")
                                               .arg(jsonParseError.errorString());
@@ -175,6 +300,7 @@ void OAuth::startAuthentication()
                         }
                     });
                 });
+                job->start();
             });
         }
     });
@@ -182,7 +308,8 @@ void OAuth::startAuthentication()
 
 void OAuth::refreshAuthentication(const QString &refreshToken)
 {
-    connect(this, &OAuth::fetchWellKnownFinished, this, [this, refreshToken] {
+    _isRefreshingToken = true;
+    auto refresh = [this, &refreshToken] {
         auto job = postTokenRequest({ { QStringLiteral("grant_type"), QStringLiteral("refresh_token") },
             { QStringLiteral("refresh_token"), refreshToken } });
         connect(job, &SimpleNetworkJob::finishedSignal, this, [this, refreshToken](QNetworkReply *reply) {
@@ -191,43 +318,63 @@ void OAuth::refreshAuthentication(const QString &refreshToken)
             QString newRefreshToken = refreshToken;
             QJsonParseError jsonParseError;
             // https://developer.okta.com/docs/reference/api/oidc/#response-properties-2
-            const QJsonObject json = QJsonDocument::fromJson(jsonData, &jsonParseError).object();
-            const QString error = json.value(QLatin1String("error")).toString();
+            const auto data = QJsonDocument::fromJson(jsonData, &jsonParseError).object().toVariantMap();
+            const QString error = data.value(QStringLiteral("error")).toString();
             if (!error.isEmpty()) {
                 if (error == QLatin1String("invalid_grant") ||
                     error == QLatin1String("invalid_request")) {
                     newRefreshToken.clear();
                 } else {
-                    qCWarning(lcOauth) << tr("Error while refreshing the token: %1 : %2").arg(error, json.value(QLatin1String("error_description")).toString());
+                    qCWarning(lcOauth) << tr("Error while refreshing the token: %1 : %2").arg(error, data.value(QStringLiteral("error_description")).toString());
                 }
             } else if (reply->error() != QNetworkReply::NoError) {
                 qCWarning(lcOauth) << tr("Error while refreshing the token: %1 : %2").arg(reply->errorString(), QString::fromUtf8(jsonData));
+                Q_EMIT refreshError(reply->error(), reply->errorString());
+                return;
             } else {
-                if (jsonParseError.error != QJsonParseError::NoError || json.isEmpty()) {
+                if (jsonParseError.error != QJsonParseError::NoError || data.isEmpty()) {
                     // Invalid or empty JSON: Network error maybe?
                     qCWarning(lcOauth) << tr("Error while refreshing the token: %1 : %2").arg(jsonParseError.errorString(), QString::fromUtf8(jsonData));
                 } else {
                     QString error;
-                    accessToken = getRequiredField(json, QStringLiteral("access_token"), &error).toString();
+                    accessToken = getRequiredField(data, QStringLiteral("access_token"), &error).toString();
                     if (!error.isEmpty()) {
                         qCWarning(lcOauth) << tr("The reply from the server did not contain all expected fields\n:%1\nReceived data: %2").arg(error, QString::fromUtf8(jsonData));
                     }
 
-                    const auto refresh_token = json.find(QStringLiteral("refresh_token"));
-                    if (refresh_token != json.constEnd() )
-                    {
+                    const auto refresh_token = data.find(QStringLiteral("refresh_token"));
+                    if (refresh_token != data.constEnd()) {
                         newRefreshToken = refresh_token.value().toString();
                     }
                 }
             }
             Q_EMIT refreshFinished(accessToken, newRefreshToken);
         });
+        job->start();
+    };
+    connect(this, &OAuth::fetchWellKnownFinished, this, [this, refresh] {
+        if (_registrationEndpoint.isValid()) {
+            auto registerJob = new RegisterClientJob(_account, _registrationEndpoint, this);
+            connect(registerJob, &RegisterClientJob::finished, this, [this, refresh](const QString &clientId, const QString &clientSecret) {
+                _clientId = clientId;
+                _clientSecret = clientSecret;
+                refresh();
+            });
+            connect(registerJob, &RegisterClientJob::errorOccured, this, [this](const QString &error) {
+                qCWarning(lcOauth) << "Failed to dynamically register the client, try the default client id" << error;
+                Q_EMIT refreshFinished(QString(), QString());
+            });
+            registerJob->start();
+        } else {
+            refresh();
+        }
     });
     fetchWellKnown();
 }
 
-void OAuth::finalize(QPointer<QTcpSocket> socket, const QString &accessToken,
-                     const QString &refreshToken, const QString &user, const QUrl &messageUrl) {
+void OAuth::finalize(const QPointer<QTcpSocket> &socket, const QString &accessToken,
+    const QString &refreshToken, const QString &user, const QUrl &messageUrl)
+{
     if (!_account->davUser().isNull() && user != _account->davUser()) {
         // Connected with the wrong user
         qCWarning(lcOauth) << "We expected the user" << _account->davUser() << "but the server answered with user" << user;
@@ -255,23 +402,19 @@ SimpleNetworkJob *OAuth::postTokenRequest(const QList<QPair<QString, QString>> &
 {
     const QUrl requestTokenUrl = _tokenEndpoint.isEmpty() ? Utility::concatUrlPath(_account->url(), QStringLiteral("/index.php/apps/oauth2/api/v1/token")) : _tokenEndpoint;
     QNetworkRequest req;
-    req.setHeader(QNetworkRequest::ContentTypeHeader, QStringLiteral("application/x-www-form-urlencoded; charset=UTF-8"));
-    const QByteArray basicAuth = QStringLiteral("%1:%2").arg(Theme::instance()->oauthClientId(), Theme::instance()->oauthClientSecret()).toUtf8().toBase64();
+    const QByteArray basicAuth = QStringLiteral("%1:%2").arg(_clientId, _clientSecret).toUtf8().toBase64();
     req.setRawHeader("Authorization", "Basic " + basicAuth);
     req.setAttribute(HttpCredentials::DontAddCredentialsAttribute, true);
 
-    auto requestBody = new QBuffer;
     QUrlQuery arguments;
-    arguments.setQueryItems(QList<QPair<QString, QString>> { { QStringLiteral("client_id"), Theme::instance()->oauthClientId() },
-                                { QStringLiteral("client_secret"), Theme::instance()->oauthClientSecret() },
+    arguments.setQueryItems(QList<QPair<QString, QString>> { { QStringLiteral("client_id"), _clientId },
+                                { QStringLiteral("client_secret"), _clientSecret },
                                 { QStringLiteral("scope"), Theme::instance()->openIdConnectScopes() } }
         << queryItems);
-
-    requestBody->setData(arguments.query(QUrl::FullyEncoded).toUtf8());
-
-    auto job = _account->sendRequest("POST", requestTokenUrl, req, requestBody);
-    job->setTimeout(qMin(30 * 1000ll, job->timeoutMsec()));
+    auto job = new SimpleNetworkJob(_account->sharedFromThis(), this);
     job->setAuthenticationJob(true);
+    job->prepareRequest("POST", requestTokenUrl, req, arguments);
+    job->setTimeout(qMin(30 * 1000ll, job->timeoutMsec()));
     return job;
 }
 
@@ -283,16 +426,6 @@ QByteArray OAuth::generateRandomString(size_t size) const
     return QByteArray(reinterpret_cast<char *>(buffer.data()), static_cast<int>(size * sizeof(quint32))).toBase64(QByteArray::Base64UrlEncoding);
 }
 
-QVariant OAuth::getRequiredField(const QJsonObject &json, const QString &s, QString *error)
-{
-    const auto out = json.constFind(s);
-    if (out == json.constEnd()) {
-        error->append(tr("\tError: Missing field %1\n").arg(s));
-        return QJsonValue();
-    }
-    return *out;
-}
-
 QUrl OAuth::authorisationLink() const
 {
     Q_ASSERT(_server.isListening());
@@ -300,8 +433,8 @@ QUrl OAuth::authorisationLink() const
     const QByteArray code_challenge = QCryptographicHash::hash(_pkceCodeVerifier, QCryptographicHash::Sha256)
                                           .toBase64(QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals);
     query.setQueryItems({ { QStringLiteral("response_type"), QStringLiteral("code") },
-        { QStringLiteral("client_id"), Theme::instance()->oauthClientId() },
-        { QStringLiteral("redirect_uri"), QStringLiteral("http://localhost:%1").arg(QString::number(_server.serverPort())) },
+        { QStringLiteral("client_id"), _clientId },
+        { QStringLiteral("redirect_uri"), QStringLiteral("%1:%2").arg(_redirectUrl, QString::number(_server.serverPort())) },
         { QStringLiteral("code_challenge"), QString::fromLatin1(code_challenge) },
         { QStringLiteral("code_challenge_method"), QStringLiteral("S256") },
         { QStringLiteral("scope"), Theme::instance()->openIdConnectScopes() },
@@ -332,54 +465,90 @@ void OAuth::authorisationLinkAsync(std::function<void (const QUrl &)> callback)
 
 void OAuth::fetchWellKnown()
 {
-    const QPair<QString, QString> urls = Theme::instance()->oauthOverrideAuthUrl();
-    if (!urls.first.isNull())
-    {
-        OC_ASSERT(!urls.second.isNull());
-        _authEndpoint = QUrl::fromUserInput(urls.first);
-        _tokenEndpoint = QUrl::fromUserInput(urls.second);
-        _wellKnownFinished = true;
-         Q_EMIT fetchWellKnownFinished();
-    }
-    else
-    {
-        QUrl wellKnownUrl = Utility::concatUrlPath(_account->url(), QStringLiteral("/.well-known/openid-configuration"));
-        QNetworkRequest req;
-        auto job = _account->sendRequest("GET", wellKnownUrl);
-        job->setFollowRedirects(false);
-        job->setAuthenticationJob(true);
-        job->setTimeout(qMin(30 * 1000ll, job->timeoutMsec()));
-        QObject::connect(job, &SimpleNetworkJob::finishedSignal, this, [this](QNetworkReply *reply) {
+    auto checkServer = new CheckServerJob(_account->sharedFromThis(), this);
+    checkServer->setTimeout(qMin(30 * 1000ll, checkServer->timeoutMsec()));
+    connect(checkServer, &CheckServerJob::instanceNotFound, this, [this](QNetworkReply *reply) {
+        if (_isRefreshingToken) {
+            Q_EMIT refreshError(reply->error(), reply->errorString());
+        } else {
+            Q_EMIT result(Error);
+        }
+    });
+    connect(checkServer, &CheckServerJob::instanceFound, this, [this] {
+        const QPair<QString, QString> urls = Theme::instance()->oauthOverrideAuthUrl();
+        if (!urls.first.isNull()) {
+            OC_ASSERT(!urls.second.isNull());
+            _authEndpoint = QUrl::fromUserInput(urls.first);
+            _tokenEndpoint = QUrl::fromUserInput(urls.second);
             _wellKnownFinished = true;
-            if (reply->error() != QNetworkReply::NoError) {
-                // Most likely the file does not exist, default to the normal endpoint
-                Q_EMIT fetchWellKnownFinished();
-                return;
-            }
-            const auto jsonData = reply->readAll();
-            QJsonParseError jsonParseError;
-            const QJsonObject json = QJsonDocument::fromJson(jsonData, &jsonParseError).object();
-
-            if (jsonParseError.error == QJsonParseError::NoError) {
-                QString authEp = json[QStringLiteral("authorization_endpoint")].toString();
-                if (!authEp.isEmpty())
-                    this->_authEndpoint = QUrl::fromEncoded(authEp.toUtf8());
-                QString tokenEp = json[QStringLiteral("token_endpoint")].toString();
-                if (!tokenEp.isEmpty())
-                    this->_tokenEndpoint = QUrl::fromEncoded(tokenEp.toUtf8());
-            } else if (jsonParseError.error == QJsonParseError::IllegalValue) {
-                qCDebug(lcOauth) << ".well-known did not return json, the server most does not support oidc";
-            } else {
-                qCWarning(lcOauth) << "Json parse error in well-known: " << jsonParseError.errorString();
-            }
             Q_EMIT fetchWellKnownFinished();
-        });
+        } else {
+            auto job = new SimpleNetworkJob(_account->sharedFromThis(), this);
+            job->setAuthenticationJob(true);
+            job->prepareRequest("GET", Utility::concatUrlPath(_account->url(), QStringLiteral("/.well-known/openid-configuration")));
+            job->setTimeout(qMin(30 * 1000ll, job->timeoutMsec()));
+            QObject::connect(job, &SimpleNetworkJob::finishedSignal, this, [this](QNetworkReply *reply) {
+                _wellKnownFinished = true;
+                if (reply->error() != QNetworkReply::NoError) {
+                    // Most likely the file does not exist, default to the normal endpoint
+                    Q_EMIT fetchWellKnownFinished();
+                    return;
+                }
+                const auto jsonData = reply->readAll();
+                QJsonParseError jsonParseError;
+                const QJsonObject json = QJsonDocument::fromJson(jsonData, &jsonParseError).object();
+                if (jsonParseError.error == QJsonParseError::NoError) {
+                    _authEndpoint = QUrl::fromEncoded(json[QStringLiteral("authorization_endpoint")].toString().toUtf8());
+                    _tokenEndpoint = QUrl::fromEncoded(json[QStringLiteral("token_endpoint")].toString().toUtf8());
+                    _registrationEndpoint = QUrl::fromEncoded(json[QStringLiteral("registration_endpoint")].toString().toUtf8());
+                    _redirectUrl = QStringLiteral("http://127.0.0.1");
+                } else if (jsonParseError.error == QJsonParseError::IllegalValue) {
+                    qCDebug(lcOauth) << ".well-known did not return json, the server most probably does not support oidc";
+                } else {
+                    qCWarning(lcOauth) << "Json parse error in well-known: " << jsonParseError.errorString();
+                }
+                Q_EMIT fetchWellKnownFinished();
+            });
+            job->start();
+        }
+    });
+    checkServer->start();
+}
+
+/**
+ * Checks whether a URL returned by the server is valid.
+ * @param url URL to validate
+ * @return true if validation is successful, false otherwise
+ */
+bool isUrlValid(const QUrl &url)
+{
+    qCDebug(lcOauth()) << "Checking URL for validity:" << url;
+
+    // we have hardcoded the oauthOverrideAuth
+    const auto overrideUrl = Theme::instance()->oauthOverrideAuthUrl();
+    if (!overrideUrl.first.isEmpty()) {
+        return QUrl::fromUserInput(overrideUrl.first).matches(url, QUrl::RemoveQuery);
     }
+
+    // the following allowlist contains URL schemes accepted as valid
+    // OAuth 2.0 URLs must be HTTPS to be in compliance with the specification
+    // for unit tests, we also permit the nonexisting oauthtest scheme
+    const QStringList allowedSchemes({ QStringLiteral("https"), QStringLiteral("oauthtest") });
+    return allowedSchemes.contains(url.scheme());
 }
 
 void OAuth::openBrowser()
 {
     authorisationLinkAsync([this](const QUrl &link) {
+        if (!isUrlValid(link)) {
+            qCWarning(lcOauth) << "URL validation failed";
+            QMetaObject::invokeMethod(qApp, "slotShowGuiMessage", Qt::QueuedConnection,
+                Q_ARG(QString, tr("Oauth2 Error")),
+                Q_ARG(QString, tr("Oauth2 authentication requires a secured connection.")));
+            emit result(Error, QString());
+            return;
+        }
+
         if (!QDesktopServices::openUrl(link)) {
             qCWarning(lcOauth) << "QDesktopServices::openUrl Failed";
             // We cannot open the browser, then we claim we don't support OAuth.
@@ -388,4 +557,4 @@ void OAuth::openBrowser()
     });
 }
 
-} // namespace OCC
+#include "oauth.moc"
