@@ -34,12 +34,13 @@
 #include <winbase.h>
 #endif
 
-#include <QStack>
-#include <QFileInfo>
+#include <QApplication>
 #include <QDir>
+#include <QFileInfo>
 #include <QLoggingCategory>
-#include <QTimer>
 #include <QObject>
+#include <QStack>
+#include <QTimer>
 #include <QTimerEvent>
 #include <qmath.h>
 
@@ -47,7 +48,6 @@ namespace OCC {
 
 Q_LOGGING_CATEGORY(lcPropagator, "sync.propagator", QtInfoMsg)
 Q_LOGGING_CATEGORY(lcDirectory, "sync.propagator.directory", QtInfoMsg)
-Q_LOGGING_CATEGORY(lcCleanupPolls, "sync.propagator.cleanuppolls", QtInfoMsg)
 
 qint64 criticalFreeSpaceLimit()
 {
@@ -109,6 +109,22 @@ PropagateItemJob::~PropagateItemJob()
     }
 }
 
+bool PropagateItemJob::scheduleSelfOrChild()
+{
+    if (_state != NotYetStarted) {
+        return false;
+    }
+    qCInfo(lcPropagator) << "Starting" << _item->_instruction << "propagation of" << _item->destination() << "by" << this;
+
+    _state = Running;
+    if (thread() != QApplication::instance()->thread()) {
+        QMetaObject::invokeMethod(this, &PropagateItemJob::start); // We could be in a different thread (neon jobs)
+    } else {
+        start();
+    }
+    return true;
+}
+
 static qint64 getMinBlacklistTime()
 {
     return qMax(qEnvironmentVariableIntValue("OWNCLOUD_BLACKLIST_TIME_MIN"),
@@ -177,13 +193,11 @@ static void blacklistUpdate(SyncJournalDb *journal, SyncFileItem &item)
 {
     SyncJournalErrorBlacklistRecord oldEntry = journal->errorBlacklistEntry(item._file);
 
-    bool mayBlacklist =
-        item._errorMayBeBlacklisted // explicitly flagged for blacklisting
-        || ((item._status == SyncFileItem::NormalError
-                || item._status == SyncFileItem::SoftError
+    const bool mayBlacklist = (item._status == SyncFileItem::NormalError)
+        || ((item._status == SyncFileItem::SoftError
                 || item._status == SyncFileItem::DetailError)
-               && item._httpErrorCode != 0 // or non-local error
-               );
+            && item._httpErrorCode != 0 // or non-local error
+        );
 
     // No new entry? Possibly remove the old one, then done.
     if (!mayBlacklist) {
@@ -270,6 +284,7 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
     case SyncFileItem::FileIgnored:
     case SyncFileItem::NoStatus:
     case SyncFileItem::BlacklistedError:
+    case SyncFileItem::Excluded:
         // nothing
         break;
     }
@@ -287,22 +302,6 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
     }
 }
 
-void PropagateItemJob::slotRestoreJobFinished(SyncFileItem::Status status)
-{
-    QString msg;
-    if (_restoreJob) {
-        msg = _restoreJob->restoreJobMsg();
-        _restoreJob->setRestoreJobMsg();
-    }
-
-    if (status == SyncFileItem::Success || status == SyncFileItem::Conflict
-        || status == SyncFileItem::Restoration) {
-        done(SyncFileItem::SoftError, msg);
-    } else {
-        done(status, tr("A file or folder was removed from a read only share, but restoring failed: %1").arg(msg));
-    }
-}
-
 // ================================================================================
 
 PropagateItemJob *OwncloudPropagator::createJob(const SyncFileItemPtr &item)
@@ -355,13 +354,23 @@ PropagateItemJob *OwncloudPropagator::createJob(const SyncFileItemPtr &item)
         } else {
             return new PropagateLocalRename(this, item);
         }
+    case CSYNC_INSTRUCTION_UPDATE_METADATA:
+        // For directories, metadata-only updates will be done after all their files are propagated.
+        if (item->isDirectory()) {
+            // Will be handled in PropagateDirectory::slotSubJobsFinished at the end
+            return nullptr;
+        }
+        return new PropagateUpdateMetaDataJob(this, item);
     case CSYNC_INSTRUCTION_IGNORE:
     case CSYNC_INSTRUCTION_ERROR:
         return new PropagateIgnoreJob(this, item);
-    default:
+    case CSYNC_INSTRUCTION_NONE:
+    case CSYNC_INSTRUCTION_STAT_ERROR:
+    case CSYNC_INSTRUCTION_EVAL_RENAME:
+    case CSYNC_INSTRUCTION_EVAL:
         return nullptr;
     }
-    return nullptr;
+    Q_UNREACHABLE();
 }
 
 qint64 OwncloudPropagator::smallFileSize()
@@ -370,10 +379,8 @@ qint64 OwncloudPropagator::smallFileSize()
     return smallFileSize;
 }
 
-void OwncloudPropagator::start(SyncFileItemVector &&items)
+void OwncloudPropagator::start(SyncFileItemSet &&items)
 {
-    Q_ASSERT(std::is_sorted(items.begin(), items.end()));
-
     /* This builds all the jobs needed for the propagation.
      * Each directory is a PropagateDirectory job, which contains the files in it.
      * In order to do that we loop over the items. (which are sorted by destination)
@@ -385,7 +392,7 @@ void OwncloudPropagator::start(SyncFileItemVector &&items)
     QVector<PropagatorJob *> directoriesToRemove;
     QString removedDirectory;
     QString maybeConflictDirectory;
-    foreach (const SyncFileItemPtr &item, items) {
+    for (const auto &item : qAsConst(items)) {
         if (!removedDirectory.isEmpty() && item->_file.startsWith(removedDirectory)) {
             // this is an item in a directory which is going to be removed.
             PropagateDirectory *delDirJob = qobject_cast<PropagateDirectory *>(directoriesToRemove.first());
@@ -446,7 +453,7 @@ void OwncloudPropagator::start(SyncFileItemVector &&items)
                 // checkForPermissions() has already run and used the permissions
                 // of the file we're about to delete to decide whether uploading
                 // to the new dir is ok...
-                foreach (const SyncFileItemPtr &item2, items) {
+                for (const auto &item2 : qAsConst(items)) {
                     if (item2->destination().startsWith(item->destination() + QLatin1Char('/'))) {
                         item2->_instruction = CSYNC_INSTRUCTION_NONE;
                         _anotherSyncNeeded = true;
@@ -465,8 +472,8 @@ void OwncloudPropagator::start(SyncFileItemVector &&items)
                 // NOTE: Currently this means that we don't update those etag at all in this sync,
                 //       but it should not be a problem, they will be updated in the next sync.
                 for (int i = 0; i < directories.size(); ++i) {
-                    if (directories[i].second->_item->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA)
-                        directories[i].second->_item->_instruction = CSYNC_INSTRUCTION_NONE;
+                    if (directories[i].second->item()->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA)
+                        directories[i].second->item()->_instruction = CSYNC_INSTRUCTION_NONE;
                 }
             } else {
                 PropagateDirectory *currentDirJob = directories.top().second;
@@ -490,7 +497,7 @@ void OwncloudPropagator::start(SyncFileItemVector &&items)
         }
     }
 
-    foreach (PropagatorJob *it, directoriesToRemove) {
+    for (auto *it : qAsConst(directoriesToRemove)) {
         _rootJob->_dirDeletionJobs.appendJob(it);
     }
 
@@ -511,50 +518,45 @@ void OwncloudPropagator::setSyncOptions(const SyncOptions &syncOptions)
     _chunkSize = syncOptions._initialChunkSize;
 }
 
-bool OwncloudPropagator::localFileNameClash(const QString &relFile)
+Result<QString, bool> OwncloudPropagator::localFileNameClash(const QString &relFile)
 {
-    const QString file(_localDir + relFile);
-    OC_ASSERT(!file.isEmpty());
-
-    if (!file.isEmpty() && Utility::fsCasePreserving()) {
-        qCDebug(lcPropagator) << "CaseClashCheck for " << file;
+    OC_ASSERT(!relFile.isEmpty());
+    if (!relFile.isEmpty() && Utility::fsCasePreserving()) {
+        const QFileInfo fileInfo(_localDir + relFile);
+        qCDebug(lcPropagator) << "CaseClashCheck for " << fileInfo.filePath();
 #ifdef Q_OS_MAC
-        const QFileInfo fileInfo(file);
         if (!fileInfo.exists()) {
-            qCWarning(lcPropagator) << "No valid fileinfo";
             return false;
         } else {
             // Need to normalize to composited form because of QTBUG-39622/QTBUG-55896
             const QString cName = fileInfo.canonicalFilePath().normalized(QString::NormalizationForm_C);
-            if (file != cName && !cName.endsWith(relFile, Qt::CaseSensitive)) {
-                qCWarning(lcPropagator) << "Detected case clash between" << file << "and" << cName;
-                return true;
+            if (fileInfo.filePath() != cName && !cName.endsWith(relFile, Qt::CaseSensitive)) {
+                qCWarning(lcPropagator) << "Detected case clash between" << fileInfo.filePath() << "and" << cName;
+                return cName;
             }
         }
 #elif defined(Q_OS_WIN)
         WIN32_FIND_DATA FindFileData;
         HANDLE hFind;
 
-        hFind = FindFirstFileW(reinterpret_cast<const wchar_t *>(FileSystem::longWinPath(file).utf16()), &FindFileData);
-        if (hFind == INVALID_HANDLE_VALUE) {
-            // returns false.
-        } else {
+        hFind = FindFirstFileW(reinterpret_cast<const wchar_t *>(FileSystem::longWinPath(fileInfo.filePath()).utf16()), &FindFileData);
+        if (hFind != INVALID_HANDLE_VALUE) {
             const QString realFileName = QString::fromWCharArray(FindFileData.cFileName);
             FindClose(hFind);
 
-            if (!file.endsWith(realFileName, Qt::CaseSensitive)) {
-                qCWarning(lcPropagator) << "Detected case clash between" << file << "and" << realFileName;
-                return true;
+            if (!fileInfo.filePath().endsWith(realFileName, Qt::CaseSensitive)) {
+                const QString clashName = fileInfo.path() + QLatin1Char('/') + realFileName;
+                qCWarning(lcPropagator) << "Detected case clash between" << fileInfo.filePath() << "and" << clashName;
+                return clashName;
             }
         }
 #else
         // On Linux, the file system is case sensitive, but this code is useful for testing.
         // Just check that there is no other file with the same name and different casing.
-        QFileInfo fileInfo(file);
         const QString fn = fileInfo.fileName();
         const QStringList list = fileInfo.dir().entryList({ fn });
         if (list.count() > 1 || (list.count() == 1 && list[0] != fn)) {
-            return true;
+            return list[0];
         }
 #endif
     }
@@ -688,18 +690,19 @@ bool OwncloudPropagator::createConflict(const SyncFileItemPtr &item,
         item->_file, Utility::qDateTimeFromTime_t(conflictModTime), conflictUserName);
     QString conflictFilePath = fullLocalPath(conflictFileName);
 
+    // If the file is locked, we want to retry this sync when it
+    // becomes available again.
+    if (FileSystem::isFileLocked(fn, FileSystem::LockMode::Exclusive)) {
+        emit seenLockedFile(fn, FileSystem::LockMode::Exclusive);
+        if (error)
+            *error = tr("File %1 is currently in use").arg(fn);
+        return false;
+    }
+
     emit touchedFile(fn);
     emit touchedFile(conflictFilePath);
-
     if (!FileSystem::rename(fn, conflictFilePath, &renameError)) {
         // If the rename fails, don't replace it.
-
-        // If the file is locked, we want to retry this sync when it
-        // becomes available again.
-        if (FileSystem::isFileLocked(fn)) {
-            emit seenLockedFile(fn);
-        }
-
         if (error)
             *error = renameError;
         return false;
@@ -749,19 +752,24 @@ QString OwncloudPropagator::adjustRenamedPath(const QString &original) const
     return OCC::adjustRenamedPath(_renamedDirectories, original);
 }
 
-bool OwncloudPropagator::updateMetadata(const SyncFileItem &item, const QString &localFolderPath, SyncJournalDb &journal, Vfs &vfs)
+Result<Vfs::ConvertToPlaceholderResult, QString> OwncloudPropagator::updatePlaceholder(const SyncFileItem &item, const QString &fileName, const QString &replacesFile)
 {
-    QString fsPath = localFolderPath + item.destination();
-    if (!vfs.convertToPlaceholder(fsPath, item)) {
-        return false;
-    }
-    auto record = item.toSyncJournalFileRecordWithInode(fsPath);
-    return journal.setFileRecord(record);
+    return syncOptions()._vfs->updateMetadata(item, fileName, replacesFile);
 }
 
-bool OwncloudPropagator::updateMetadata(const SyncFileItem &item)
+Result<Vfs::ConvertToPlaceholderResult, QString> OwncloudPropagator::updateMetadata(const SyncFileItem &item)
 {
-    return updateMetadata(item, _localDir, *_journal, *syncOptions()._vfs);
+    const QString fsPath = fullLocalPath(item.destination());
+    const auto result = updatePlaceholder(item, fsPath, {});
+    if (!result || result.get() != Vfs::ConvertToPlaceholderResult::Ok) {
+        return result;
+    }
+    const auto record = item.toSyncJournalFileRecordWithInode(fsPath);
+    const auto dBresult = _journal->setFileRecord(record);
+    if (!dBresult) {
+        return dBresult.error();
+    }
+    return Vfs::ConvertToPlaceholderResult::Ok;
 }
 
 // ================================================================================
@@ -836,9 +844,9 @@ bool PropagatorCompositeJob::scheduleSelfOrChild()
 
     // Now it's our turn, check if we have something left to do.
     // First, convert a task to a job if necessary
-    while (_jobsToDo.isEmpty() && !_tasksToDo.isEmpty()) {
-        SyncFileItemPtr nextTask = _tasksToDo.first();
-        _tasksToDo.remove(0);
+    while (_jobsToDo.empty() && !_tasksToDo.empty()) {
+        const SyncFileItemPtr nextTask = *_tasksToDo.begin();
+        _tasksToDo.erase(_tasksToDo.begin());
         PropagatorJob *job = propagator()->createJob(nextTask);
         if (!job) {
             qCWarning(lcDirectory) << "Useless task found for file" << nextTask->destination() << "instruction" << nextTask->_instruction;
@@ -857,10 +865,10 @@ bool PropagatorCompositeJob::scheduleSelfOrChild()
 
     // If neither us or our children had stuff left to do we could hang. Make sure
     // we mark this job as finished so that the propagator can schedule a new one.
-    if (_jobsToDo.isEmpty() && _tasksToDo.isEmpty() && _runningJobs.isEmpty()) {
+    if (_jobsToDo.isEmpty() && _tasksToDo.empty() && _runningJobs.isEmpty()) {
         // Our parent jobs are already iterating over their running jobs, post to the event loop
         // to avoid removing ourself from that list while they iterate.
-        QMetaObject::invokeMethod(this, "finalize", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &PropagatorCompositeJob::finalize, Qt::QueuedConnection);
     }
     return false;
 }
@@ -886,7 +894,7 @@ void PropagatorCompositeJob::slotSubJobFinished(SyncFileItem::Status status)
         _hasError = status;
     }
 
-    if (_jobsToDo.isEmpty() && _tasksToDo.isEmpty() && _runningJobs.isEmpty()) {
+    if (_jobsToDo.isEmpty() && _tasksToDo.empty() && _runningJobs.isEmpty()) {
         finalize();
     } else {
         propagator()->scheduleNextJob();
@@ -907,7 +915,7 @@ void PropagatorCompositeJob::finalize()
 qint64 PropagatorCompositeJob::committedDiskSpace() const
 {
     qint64 needed = 0;
-    foreach (PropagatorJob *job, _runningJobs) {
+    for (auto *job : _runningJobs) {
         needed += job->committedDiskSpace();
     }
     return needed;
@@ -916,8 +924,7 @@ qint64 PropagatorCompositeJob::committedDiskSpace() const
 // ================================================================================
 
 PropagateDirectory::PropagateDirectory(OwncloudPropagator *propagator, const SyncFileItemPtr &item)
-    : PropagatorJob(propagator)
-    , _item(item)
+    : PropagateItemJob(propagator, item)
     , _firstJob(propagator->createJob(item))
     , _subJobs(propagator)
 {
@@ -973,8 +980,7 @@ void PropagateDirectory::slotFirstJobFinished(SyncFileItem::Status status)
         if (_state != Finished) {
             // Synchronously abort
             abort(AbortType::Synchronous);
-            _state = Finished;
-            emit finished(status);
+            done(status);
         }
         return;
     }
@@ -982,7 +988,7 @@ void PropagateDirectory::slotFirstJobFinished(SyncFileItem::Status status)
     propagator()->scheduleNextJob();
 }
 
-void PropagateDirectory::slotSubJobsFinished(SyncFileItem::Status status)
+void PropagateDirectory::slotSubJobsFinished(const SyncFileItem::Status status)
 {
     if (!_item->isEmpty() && status == SyncFileItem::Success) {
         // If a directory is renamed, recursively delete any stale items
@@ -1001,16 +1007,20 @@ void PropagateDirectory::slotSubJobsFinished(SyncFileItem::Status status)
         // For new directories we always want to update the etag once
         // the directory has been propagated. Otherwise the directory
         // could appear locally without being added to the database.
-        if (_item->_instruction == CSYNC_INSTRUCTION_RENAME
-            || _item->_instruction == CSYNC_INSTRUCTION_NEW
-            || _item->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA) {
-            if (!propagator()->updateMetadata(*_item)) {
-                status = _item->_status = SyncFileItem::FatalError;
-                _item->_errorString = tr("Error writing metadata to the database");
-                qCWarning(lcDirectory) << "Error writing to the database for file" << _item->_file;
+        if (_item->_instruction & (CSYNC_INSTRUCTION_RENAME | CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_UPDATE_METADATA)) {
+            const auto result = propagator()->updateMetadata(*_item);
+            if (!result) {
+                qCWarning(lcDirectory) << "Error writing to the database for file" << _item->_file << "with" << result.error();
+                done(SyncFileItem::FatalError, tr("Error updating metadata: %1").arg(result.error()));
+                return;
+            } else if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
+                done(SyncFileItem::SoftError, tr("%1 the folder is currently in use").arg(_item->destination()));
+                return;
             }
         }
     }
+    // don't call done, we only propagate the state of the child items
+    Q_ASSERT(_state != Finished);
     _state = Finished;
     emit finished(status);
 }
@@ -1102,55 +1112,6 @@ void PropagateRootDirectory::slotDirDeletionJobsFinished(SyncFileItem::Status st
 
 // ================================================================================
 
-CleanupPollsJob::~CleanupPollsJob()
-{
-}
-
-void CleanupPollsJob::start()
-{
-    if (_pollInfos.empty()) {
-        emit finished();
-        deleteLater();
-        return;
-    }
-
-    auto info = _pollInfos.first();
-    _pollInfos.pop_front();
-    SyncJournalFileRecord record;
-    SyncFileItemPtr item(new SyncFileItem);
-    item->_file = info._file;
-    item->_modtime = info._modtime;
-    item->_size = info._fileSize;
-    PollJob *job = new PollJob(_account, info._url, item, _journal, _localPath, this);
-    connect(job, &PollJob::finishedSignal, this, &CleanupPollsJob::slotPollFinished);
-    job->start();
-}
-
-void CleanupPollsJob::slotPollFinished()
-{
-    PollJob *job = qobject_cast<PollJob *>(sender());
-    OC_ASSERT(job);
-    if (job->_item->_status == SyncFileItem::FatalError) {
-        emit aborted(job->_item->_errorString);
-        deleteLater();
-        return;
-    } else if (job->_item->_status != SyncFileItem::Success) {
-        qCWarning(lcCleanupPolls) << "There was an error with file " << job->_item->_file << job->_item->_errorString;
-    } else {
-        if (!OwncloudPropagator::updateMetadata(*job->_item, _localPath, *_journal, *_vfs)) {
-            qCWarning(lcCleanupPolls) << "database error";
-            job->_item->_status = SyncFileItem::FatalError;
-            job->_item->_errorString = tr("Error writing metadata to the database");
-            emit aborted(job->_item->_errorString);
-            deleteLater();
-            return;
-        }
-        _journal->setUploadInfo(job->_item->_file, SyncJournalDb::UploadInfo());
-    }
-    // Continue with the next entry, or finish
-    start();
-}
-
 QString OwncloudPropagator::fullRemotePath(const QString &tmp_file_name) const
 {
     // TODO: should this be part of the _item (SyncFileItemPtr)?
@@ -1161,4 +1122,47 @@ QString OwncloudPropagator::remotePath() const
 {
     return _remoteFolder;
 }
+
+PropagateUpdateMetaDataJob::PropagateUpdateMetaDataJob(OwncloudPropagator *propagator, const SyncFileItemPtr &item)
+    : PropagateItemJob(propagator, item)
+{
+}
+
+void OCC::PropagateUpdateMetaDataJob::start()
+{
+    // Update the database:  New remote fileid or Etag or RemotePerm
+    // Or for files that were detected as "resolved conflict".
+    // Or a local inode/mtime change
+
+    // In case of "resolved conflict": there should have been a conflict because they
+    // both were new, or both had their local mtime or remote etag modified, but the
+    // size and mtime is the same on the server.  This typically happens when the
+    // database is removed. Nothing will be done for those files, but we still need
+    // to update the database.
+
+    const QString filePath = propagator()->fullLocalPath(_item->destination());
+    if (_item->_direction == SyncFileItem::Down) {
+        // If the 'W' remote permission changed, update the local filesystem
+        SyncJournalFileRecord prev;
+        if (propagator()->_journal->getFileRecord(_item->_file, &prev)
+            && prev.isValid()
+            && prev._remotePerm.hasPermission(RemotePermissions::CanWrite) != _item->_remotePerm.hasPermission(RemotePermissions::CanWrite)) {
+            const bool isReadOnly = !_item->_remotePerm.isNull() && !_item->_remotePerm.hasPermission(RemotePermissions::CanWrite);
+            FileSystem::setFileReadOnlyWeak(filePath, isReadOnly);
+            if (_item->_checksumHeader.isEmpty()) {
+                _item->_checksumHeader = prev._checksumHeader;
+            }
+            _item->_serverHasIgnoredFiles |= prev._serverHasIgnoredFiles;
+        }
+    }
+    const auto result = propagator()->updateMetadata(*_item);
+    if (!result) {
+        done(SyncFileItem::FatalError, tr("Could not update file : %1").arg(result.error()));
+        return;
+    } else if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
+        done(SyncFileItem::SoftError, tr("The file %1 is currently in use").arg(_item->_file));
+        return;
+    }
+    done(SyncFileItem::Success);
+}
 }
